﻿;==================================================================================================
;AUTHOR   : https://www.xyplorer.com/xyfc/viewtopic.php?t=25763
;Function : Rock-click (Left-Mouse-Down + Right-Mouse-Click) run custom queue menus (Supports Desktop, Explorer, XYplorer)
;Created  : 2023-02-28
;Modified : 2023-04-22
;Version  : v1.2.1QS
;==================================================================================================
#Requires AutoHotkey v2.0.2
#HotIf MouseIsOver("ahk_class Progman") || MouseIsOver("ahk_class WorkerW") || MouseIsOver("ahk_class CabinetWClass") || MouseIsOver("ahk_class ThunderRT6FormDC")


MouseIsOver(WinTitle) {
    MouseGetPos ,, &Win
	return WinExist(WinTitle " ahk_id " Win)
}


; Get our own HWND (we have no visible window)
G_OwnHWND := A_ScriptHwnd + 0


; Get messages back from XYplorer
OnMessage(0x4a, Receive_WM_COPYDATA)

xyPath  := "D:\XYplorer\XYplorer.exe"      ; XY path
if !FileExist(xyPath)
{
   TrayTip xyPath, "XYplorer path does not exist!"
   ;ExitApp
}
exHwnd  := ""
selMenu:="", selPos:="", selItem:=""






;==================================================================================================
; Dark theme
;==================================================================================================

; Dark mode (Required Windows 10 version)
uxtheme := DllCall("GetModuleHandle", "str", "uxtheme", "ptr")
SetPreferredAppMode := DllCall("GetProcAddress", "ptr", uxtheme, "ptr", 135, "ptr")
FlushMenuThemes := DllCall("GetProcAddress", "ptr", uxtheme, "ptr", 136, "ptr")
DllCall(SetPreferredAppMode, "int", 1) ; Dark
DllCall(FlushMenuThemes)

; Switch from dark to not dark
/*
DllCall(SetPreferredAppMode, "int", 0) ; NOT Dark
DllCall(FlushMenuThemes)
*/



;==================================================================================================
; Add menu items, the format is   Name|XY script|Icon file
;==================================================================================================

menuItems := '
    (
	 Focus in XYplorer|Focus in XYplorer|.\Icons\1.ico
	 2|echo "2"
	 3|echo "3"
	 4|echo "4"
	 5|echo "5"
	)'






;==================================================================================================
; Create a popup menu
;==================================================================================================

; Create main menu
 MyMenu := Menu()
 ;MyMenu.Name := "Main menu"                                   ; Used to show which menu is selected

 menuItemsArray := strSplit(menuItems, "`n")
 for index, menuItem in menuItemsArray
 {
    itemsArray := strSplit(trim(menuItem), "|")
    for index2, itemApart in itemsArray
    {
	   switch index2
	   {
	      case "1":
              if itemApart != "-"                                             ; Add an item to the menu, or add a separator
	             MyMenu.Add itemApart, MenuHandler
			  else
			     MyMenu.Add

          case "3":
              if FileExist(itemApart)
                 MyMenu.SetIcon(itemsArray[1], itemApart)                     ; Add icon if icon file exists
	   }
    }
 }

 MyMenu.Default := "Focus in XYplorer"

; Global selected
MenuHandler(Item, ItemPos, MyMenu) {
	try IDLast := WinGetIDLast("ahk_class #32768")
    catch
		return
    if WinExist("ahk_id " IDLast)                         ; Automatically close another menu
       WinClose

	;MsgBox "You selected " Item " (position " ItemPos " in " MyMenu.Name ")"
	global selPos:=ItemPos, selItem := trim(menuItemsArray[ItemPos])   ; Global varibles
}






;==================================================================================================
; Rock-click (Left-Mouse-Down + Right-Mouse-Click) run custom menu
;==================================================================================================

~LButton & RButton Up::      ; Left-Mouse-Down + Right-Mouse-Click
{
 ctrl := ""
 if WinActive("ahk_class ThunderRT6FormDC")                          ; If in the XY window
 {
    MouseGetPos(&fX, &fY, &id, &ctrl)

    switch ctrl                                                      ; Floating preview does not support send script, run script via XYplorer.exe
    {
       case "ThunderRT6PictureBoxDC3":
	       ctrl := "FP"
	   default:
	       ctrls := WinGetControls(id)
           if ctrls.length > 20
              ctrl := true
           else
              ctrl := "FP"
    }

    MyMenu.Disable("Focus in XYplorer")

    WinGetPos(&dX, &dY, &dW, &dH, "ahk_class ThunderRT6FormDC")

    If(fX > 300 && fX < dW-235 && fY > 100 && fY < dH-60) {          ; Hover area: 300 is the width of the tree and 235 is the width of the Catalog or Preview pane, 100 is the height above the column header and 60 is the height of the status bar
	   run ".\menu2.ahk " ctrl                                       ; Display menu2 after the first menu displayed
	   sleep 200                                                     ; Try to display menus synchronously
	   Send_WM_COPYDATA2("Ready", "menu2.ahk ahk_class AutoHotkey")  ; Send message to display menu2
       ;xyHwnd := GetxyHwnd()                                         ; Send the script to XY (Display XY menu #1400)
	   ;Send_WM_COPYDATA(xyHwnd, '::#1400')
	   CoordMode("menu")
	   MyMenu.Show(800, 200)                                         ; Display first menu at

	   if WinExist("ahk_class #32768") {                             ; Automatically close another menu
	      sleep 200
	      try IDLast := WinGetIDLast("ahk_class #32768")
		  catch
		      return
          MouseGetPos , , &id
          if WinGetClass("ahk_id " id) == "#32768"
		     send "{LButton}"

          if(WinExist("ahk_class " IDLast) && WinGetProcessName(WinActivate("A")) == "XYplorer.exe")
	         WinClose "ahk_class " IDLast
	      else if(WinExist("ahk_id " IDLast) && WinExist("ahk_id " id) && WinGetClass("ahk_id " id) != "#32768")
	         WinClose "ahk_id " IDLast
	      else if(WinExist("ahk_id " IDLast) && id == IDLast)
	         WinClose "ahk_id " IDLast
	   }
	} else Send("{RButton}")
	MyMenu.Enable("Focus in XYplorer")
 } else ;if WinActive("ahk_class Progman") || WinActive("ahk_class CabinetWClass")      ; If on the desktop, Explorer, pop up the custom menu
 {
    MouseGetPos ,, &Win
    global exHwnd := Win || WinExist('A')                            ; Get HWND of the active window
	global selected := explorerGetSel(exHwnd)                        ; Get desktop, explorer selected items

	run ".\menu2.ahk" " " selected                                   ; Display menu2 after the first menu displayed
    sleep 200                                                        ; Try to display menus synchronously
	Send_WM_COPYDATA2("Ready", "menu2.ahk ahk_class AutoHotkey")     ; Send message to display menu2
    CoordMode "Menu"
    MyMenu.Show()                                                    ; Display menu

	if WinExist("ahk_class #32768") {                                ; Automatically close another menu
	   sleep 200
	   try IDLast := WinGetIDLast("ahk_class #32768")
	   catch
		   return
       MouseGetPos , , &id
       if WinGetClass("ahk_id " id) == "#32768"
		  send "{LButton}"
	   else if(WinExist("ahk_id " IDLast) && WinExist("ahk_id " id) && WinGetClass("ahk_id " id) != "#32768")
	      WinClose "ahk_id " IDLast
	   else if(WinExist("ahk_id " IDLast) && id == IDLast)
	      WinClose "ahk_id " IDLast
	}
 }

if !selItem                 ; Return if no menu item is selected
    return






;==================================================================================================
; Perform different actions
;==================================================================================================

 selArray := strSplit(selItem, "|")
 if selArray[1] = "Focus in XYplorer"
    selItem := "Focus in XYplorer"

 switch selItem
 {
    case "Focus in XYplorer":
		if(!selected || selected == "ERROR")
		{
		   TrayTip "Nothing selected!", , 1
		   return
		}

        if (xyHwnd := GetxyHwnd()) && (ctrl != "FP") {                                                                                         ; If xy is running and not in the Floating Preview
		    WinActivate("ahk_id " xyHwnd)                                                                                                      ; Active XY window
            Send_WM_COPYDATA(xyHwnd, '::tab`("new"`, gpc`("' selected '"`, "path"`)`)`; selectitems "' selected '"')                           ; Send the script to XY
        } else {
            Run xyPath ' /feed=|::tab("new"`, gpc("' selected '"`, "path"))`; goto "' selected '"|'                                            ; If XY is not running, run XY and run the script
			;WinWaitActive WinExist("ahk_class ThunderRT6FormDC")                                                                               ; Waiting to activate the window
			;sleep 3000
			;WinActivate("ahk_class ThunderRT6FormDC")
        }
    default:
		if selArray.Length > 1 {
		   selItem1 := selArray[1]
		   selItem2 := selArray[2]
		   MsgBox "You selected " selItem1 " (Value is " selItem2 ") (position " selPos " in " selMenu ")"
		}
 }

 global selMenu:="", selPos:="", selItem:=""
 return

}






;==================================================================================================
; Desktop, Explore get functions
;==================================================================================================

; Get the selected items, excluding shortcut item on the desktop
/*
 selNoDeskLnk := explorerGet(exHwnd,true)
 if(selNoDeskLnk != "ERROR" && selNoDeskLnk)
    msgbox selNoDeskLnk



; Get the selected items
 explorerSel := explorerGetSel(exHwnd)
 if(explorerSel != "ERROR" && explorerSel)
    msgbox explorerSel
*/

explorerGetSel(exHwnd, selection := True)
{
    hwWindow := ''
    ret := ""
    Switch window := explorerGetWindow(exHwnd)
	{
     Case '':
	       Return 'ERROR'
     Case 'desktop':
         Try hwWindow := ControlGetHwnd('SysListView321', 'ahk_class Progman')
         hwWindow := hwWindow || ControlGetHwnd('SysListView321', 'ahk_id ' exHwnd)
         Loop Parse ListViewGetContent((selection ? 'Selected' : '') ' Col1', hwWindow), '`n', '`r'
		 {
		  SplitPath A_LoopField, , , &ext                            ; Get ext
		  if(!ext && !DirExist(A_Desktop '\' A_LoopField))           ; Get the target of the shortcut, and stack to the variable ret, if there is no ext and not a folder
		  {
		     shortcut := A_Desktop '\' A_LoopField ".lnk"
			 if !fileExist(shortcut)
			    shortcut := A_DesktopCommon '\' A_LoopField ".lnk"
		     FileGetShortcut shortcut, &OutTarget
			 ret .= OutTarget '`n'
		  } else {                                                   ; If it is a file or a folder, stack to the varibles ret
             ret .= A_Desktop '\' A_LoopField '`n'
		  }
		 }
     Default:                                                        ; If it is not on the desktop, stack to the variable ret
         For item in selection ? window.document.selectedItems : window.document.Folder.Items
         ret .= item.path '`n'
    }

    Return Trim(ret, '`n')
}



; All items, excluding desktop shortcut items
/*
 items := explorerGet(exHwnd, false)
 if(items != "ERROR" && items)
    msgbox items
*/

explorerGet(exHwnd,selection:=false)
{
    hwWindow := ''
    ret := ""
    Switch window := explorerGetWindow(exHwnd)
	{
     Case '':
	       Return 'ERROR'
     Case 'desktop':
         Try hwWindow := ControlGetHwnd('SysListView321', 'ahk_class Progman')
         hwWindow := hwWindow || ControlGetHwnd('SysListView321', 'ahk_id ' exHwnd)
         Loop Parse ListViewGetContent((selection ? 'Selected' : '') ' Col1', hwWindow), '`n', '`r'
		 {
		  If FileExist(A_Desktop '\' A_LoopField) ; ignore special icons like Computer (at least for now)
          ret .= A_Desktop '\' A_LoopField '`n'
		 }
     Default:
         For item in selection ? window.document.selectedItems : window.document.Folder.Items
         ret .= item.path '`n'
    }

    Return Trim(ret, '`n')
}



; If it is on the desktop
explorerGetWindow(exHwnd)
{
 class := WinGetClass(exHwnd := exHwnd || WinExist('A'))
 Switch
 {
  Case WinGetProcessName(exHwnd) != 'explorer.exe': Return
  Case class ~= 'Progman|WorkerW': Return 'desktop'
  Case class ~= '(Cabinet|Explore)WClass':
   For window in ComObject('Shell.Application').Windows
    Try If window.hwnd = exHwnd
     Return window
 }
}






;==================================================================================================
; XYplorer Send and Get functions
;==================================================================================================

XY_Get(bAll:=false, bSelection:=false)
{
    xyQueryScript := '::if (!' bAll ' && !' bSelection ') {$return = "<curpath>"`;} elseif (' bAll ') {$return = listpane(, , , "<crlf>")`;} elseif (' bSelection ') {$return = get("SelectedItemsPathNames", "<crlf>")`;} copydata ' G_OwnHWND ', "$return", 2`;'

    if xyHwnd := GetxyHwnd()
       Send_WM_COPYDATA(xyHwnd, xyQueryScript)

    return dataReceived
}


GetxyHwnd() {
	static xyClass := 'ahk_class ThunderRT6FormDC'

	if hwnd := WinActive(xyClass)
		return hwnd
	else if WinExist(xyClass)
		return WinGetList(xyClass)[1]
}


Send_WM_COPYDATA(xyHwnd, message) {
   if !(xyHwnd)
       return

   size := StrLen(message)

   COPYDATA := Buffer(A_PtrSize * 3)
   NumPut("Ptr", 4194305, COPYDATA, 0)
   NumPut("UInt", size * 2, COPYDATA, A_PtrSize)
   NumPut("Ptr", StrPtr(message), COPYDATA, A_PtrSize * 2)

   return DllCall("User32.dll\SendMessageW", "Ptr", xyHwnd, "UInt", 74, "Ptr", 0, "Ptr", COPYDATA, "Ptr")
}


Receive_WM_COPYDATA(wParam, lParam, *) {
	global dataReceived := StrGet(
		NumGet(lParam + 2 * A_PtrSize, 'Ptr'),   ; COPYDATASTRUCT.lpData, ptr to a str presumably
		NumGet(lParam + A_PtrSize, 'UInt') / 2   ; COPYDATASTRUCT.cbData, count bytes of lpData, /2 to get count chars in unicode str
	)
}






;==================================================================================================
; Send message to display next menu
;==================================================================================================

Send_WM_COPYDATA2(StringToSend, TargetScriptTitle)
{
    CopyDataStruct := Buffer(3*A_PtrSize)  ; Set up the structure's memory area.
    SizeInBytes := (StrLen(StringToSend) + 1) * 2
    NumPut( "Ptr", SizeInBytes, "Ptr", StrPtr(StringToSend), CopyDataStruct, A_PtrSize)
    Prev_DetectHiddenWindows := A_DetectHiddenWindows
    Prev_TitleMatchMode := A_TitleMatchMode
    DetectHiddenWindows True
    SetTitleMatchMode 2
    TimeOutTime := 10  ; Optional. Milliseconds to wait for response from receiver.ahk. Default is 5000
    try RetValue := SendMessage(0x004A, 0, CopyDataStruct,, TargetScriptTitle,,,, TimeOutTime) ; 0x004A is WM_COPYDATA.
	catch
	    return
    DetectHiddenWindows Prev_DetectHiddenWindows  ; Restore original setting for the caller.
    SetTitleMatchMode Prev_TitleMatchMode         ; Same.
    return RetValue  ; Return SendMessage's reply back to our caller.
}